@@ -836,7 +836,7 @@ public abstract class CQLTester
 
     protected ResultMessage.Prepared prepare(String query) throws Throwable
     {
-        return QueryProcessor.prepare(formatQuery(query), ClientState.forInternalCalls(), false);
+        return QueryProcessor.instance.prepare(formatQuery(query), ClientState.forInternalCalls(), false);
     }
 
     protected UntypedResultSet execute(String query, Object... values) throws Throwable
